Files
sgl-project--sglang/python/sglang/srt/layers/attention/dsv4/quant_k_cache.py
T
wehub-resource-sync 94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
chore: import upstream snapshot with attribution
2026-07-13 12:38:16 +08:00

121 lines
3.6 KiB
Python

import torch
import triton
import triton.language as tl
from sglang.srt.layers.attention.dsv4.index_buf_accessor import NopeFp8RopeBf16Pack
from sglang.srt.layers.quantization.fp8_kernel import is_fp8_fnuz
fp8_dtype = torch.float8_e4m3fnuz if is_fp8_fnuz() else torch.float8_e4m3fn
@triton.jit
def _quant_k_cache_fused_kernel(
k_bf16_ptr,
k_nope_fp8_ptr,
k_rope_bf16_ptr,
scale_k_nope_uint8_ptr,
k_bf16_stride_0,
k_nope_fp8_stride_0,
k_rope_bf16_stride_0,
scale_stride_0,
DIM_NOPE: tl.constexpr,
DIM_ROPE: tl.constexpr,
TILE_SIZE: tl.constexpr,
NUM_TILES: tl.constexpr,
FP8_MIN: tl.constexpr,
FP8_MAX: tl.constexpr,
EPS: tl.constexpr,
):
token_id = tl.program_id(0)
tile_id = tl.program_id(1)
if tile_id == NUM_TILES:
rope_range = tl.arange(0, TILE_SIZE)
rope_mask = rope_range < DIM_ROPE
in_rope_offsets = token_id * k_bf16_stride_0 + DIM_NOPE + rope_range
rope_data = tl.load(k_bf16_ptr + in_rope_offsets, mask=rope_mask, other=0.0)
out_rope_offsets = token_id * k_rope_bf16_stride_0 + rope_range
tl.store(k_rope_bf16_ptr + out_rope_offsets, rope_data, mask=rope_mask)
else:
tile_range = tl.arange(0, TILE_SIZE)
in_tile_offsets = token_id * k_bf16_stride_0 + tile_id * TILE_SIZE + tile_range
x_bf16 = tl.load(k_bf16_ptr + in_tile_offsets)
x_fp32 = x_bf16.to(tl.float32)
abs_x = tl.abs(x_fp32)
max_abs = tl.max(abs_x)
max_abs_clamped = tl.maximum(max_abs, EPS)
scale = max_abs_clamped / FP8_MAX
log2_scale = tl.log2(scale)
ceil_log2 = tl.math.ceil(log2_scale)
scale_pow2_fp32 = tl.exp2(ceil_log2)
scale_inv = 1.0 / scale_pow2_fp32
x_scaled = x_fp32 * scale_inv
x_fp8 = tl.clamp(x_scaled, FP8_MIN, FP8_MAX).to(k_nope_fp8_ptr.dtype.element_ty)
out_fp8_offsets = (
token_id * k_nope_fp8_stride_0 + tile_id * TILE_SIZE + tile_range
)
tl.store(k_nope_fp8_ptr + out_fp8_offsets, x_fp8)
exponent = ceil_log2.to(tl.int32)
scale_uint8 = (exponent + 127).to(tl.uint8)
out_scale_offset = token_id * scale_stride_0 + tile_id
tl.store(scale_k_nope_uint8_ptr + out_scale_offset, scale_uint8)
def quant_to_nope_fp8_rope_bf16_pack_triton(
k_bf16: torch.Tensor,
) -> NopeFp8RopeBf16Pack:
assert k_bf16.dtype == torch.bfloat16
num_tokens, hidden_dim = k_bf16.shape
assert hidden_dim == 512
dim_nope = 448
dim_rope = 64
tile_size = 64
num_tiles = dim_nope // tile_size
k_bf16 = k_bf16.contiguous()
k_nope_fp8 = torch.empty(
(num_tokens, dim_nope), dtype=fp8_dtype, device=k_bf16.device
)
k_rope_bf16 = torch.empty(
(num_tokens, dim_rope), dtype=torch.bfloat16, device=k_bf16.device
)
scale_k_nope_ue8m0 = torch.empty(
(num_tokens, num_tiles), dtype=torch.uint8, device=k_bf16.device
)
fp8_dtype_info = torch.finfo(fp8_dtype)
grid = (num_tokens, num_tiles + 1)
_quant_k_cache_fused_kernel[grid](
k_bf16,
k_nope_fp8,
k_rope_bf16,
scale_k_nope_ue8m0,
k_bf16.stride(0),
k_nope_fp8.stride(0),
k_rope_bf16.stride(0),
scale_k_nope_ue8m0.stride(0),
DIM_NOPE=dim_nope,
DIM_ROPE=dim_rope,
TILE_SIZE=tile_size,
NUM_TILES=num_tiles,
FP8_MIN=fp8_dtype_info.min,
FP8_MAX=fp8_dtype_info.max,
EPS=1e-8,
)
return NopeFp8RopeBf16Pack(
k_nope_fp8=k_nope_fp8,
k_rope_bf16=k_rope_bf16,
scale_k_nope_ue8m0=scale_k_nope_ue8m0,
)